AchScreenshotter2 = LibStub("AceAddon-3.0"):NewAddon("AchScreenshotter2", "AceEvent-3.0", "AceTimer-3.0")

local debug = false
--[===[@alpha@
debug = true
--@end-alpha@]===]
local L = LibStub("AceLocale-3.0"):GetLocale("AchScreenshotter2", not debug)

local mod = AchScreenshotter2
local addonname = "AchScreenShotter2"
local screenshot_source = nil
local screenshot_hideui = false

local function profsetup()
	local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(AchScreenshotter2.db)
	return profiles
end

function AchScreenshotter2:OnInitialize()
  mod.db = LibStub("AceDB-3.0"):New("AchScreenshotter2DB", mod:getdefaults(), true)
  mod.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
  mod.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
  mod.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
  mod.db.RegisterCallback(self, "OnProfileDeleted", "RefreshConfig")
  mod.sv = mod.db.profile
  mod.gsv = mod.db.global
end

function AchScreenshotter2:OnEnable()
  mod:register_events()
  
  if LibStub:GetLibrary('LibDataBroker-1.1', true) then
    mod:LoadDataBrokerPlugin()
  end

  mod.ACR = LibStub("AceConfigRegistry-3.0")
  mod.ACR:RegisterOptionsTable(addonname, profsetup)
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Main"], mod:getmainoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Achievement"], mod:getachievementoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Level"], mod:getleveloptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Reputation"], mod:getreputationoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Battleground"], mod:getbattlegroundoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Dungeon"], mod:getdungeonoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Boss"], mod:getbossoptions())
  mod.ACR:RegisterOptionsTable(addonname.." "..L["Statistic"], mod:getstatisticoptions())

  mod.ACD = LibStub("AceConfigDialog-3.0")
  mod.ACD:AddToBlizOptions(addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Main"], L["Main"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Achievement"], L["Achievement"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Level"], L["Level"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Reputation"], L["Reputation"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Battleground"], L["Battleground"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Dungeon"], L["Dungeon"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Boss"], L["Boss"], addonname)
  mod.ACD:AddToBlizOptions(addonname.." "..L["Statistic"], L["Statistic"], addonname)
end

-- event registers

-- @usage setzt die screenshot events
local function register_screenshot_events()
  mod:RegisterEvent("SCREENSHOT_FAILED", "screen_failed")
  mod:RegisterEvent("SCREENSHOT_SUCCEEDED", "screen_succ")
end

-- @usage setzt die events auf die reagiert werden soll
function AchScreenshotter2:register_events()
  if mod.sv.enable then
    if mod.sv.achievement.enable then
      mod:RegisterEvent("ACHIEVEMENT_EARNED", "achievement_function")
    end
    if mod.sv.levelup.enable then
      mod:RegisterEvent("PLAYER_LEVEL_UP", "levelup_function")
    end
    if mod.sv.reputation.enable then
      mod:RegisterEvent("CHAT_MSG_SYSTEM", "reputation_function")
    end
    if mod.sv.bg.enable then
      mod:RegisterEvent("UPDATE_BATTLEFIELD_STATUS", "battlefield_function")
    end
    if mod.sv.dungeon.enable then
      mod:RegisterEvent("LFG_COMPLETION_REWARD", "dungeon_function")
    end
    if mod.sv.boss.enable then
      mod:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "boss_function")
    end
  end
end

-- functions

-- @usage versteckt das ui oder nicht und macht den screenshot
function AchScreenshotter2:screenshot()
  if screenshot_hideui then
    UIParent:Hide()
  end
  Screenshot()
end

-- @usage nimmt einen screenshot nach "delay" auf
-- @usage loescht davor noch alle events damit nur ein screenshot aufgenommen wird
function mod:take_screenshot(source, hide, delay)
  if (screenshot_source == nil) then -- damit nur ein event gleichzeitig einen screenshot ausloest
    screenshot_source = source
    screenshot_hideui = hide
    mod:UnregisterAllEvents()
    register_screenshot_events()
    mod:ScheduleTimer("screenshot", delay)
  end
end

-- @usage aktiviert nach "waittime" wieder die events
local function reenable()
  mod:UnregisterAllEvents()
  mod:ScheduleTimer("register_events", mod.sv.waittime)
end


-- @usage erneuert die konfiguration und setzt dann alle events neu
function AchScreenshotter2:RefreshConfig()
  mod.sv = mod.db.profile
  mod:UnregisterAllEvents()
  mod:register_events()
end

-- event functions

-- @usage wenn der screen nicht erfolgreich war gibt er es auch und ruft reenable() auf
function AchScreenshotter2:screen_failed()
  if screenshot_hideui then
    UIParent:Show()
    screenshot_hideui =  false
  end
  mod:print(L["ScreenShoot failed"])
  if mod.sv.statistic.enable then
    mod.sv.statistic.fails = mod.sv.statistic.fails + 1
  end
  screenshot_source = nil
  reenable()
end

-- @usage wenn der screen erfolgreich war wird reenable() aufgerufen
function AchScreenshotter2:screen_succ()
  if screenshot_hideui then
    UIParent:Show()
    screenshot_hideui =  false
  end
  if mod.sv.statistic.enable then
    mod.sv.statistic[screenshot_source] = mod.sv.statistic[screenshot_source] + 1
  end
  screenshot_source = nil
  reenable()
end

function AchScreenshotter2:getdefaults()
  local defaults = {
        profile = {
          enable = true,
          delay = 0.5,
          waittime = 30,
          achievement = {
            enable = true,
            hideui = false,
            },
          levelup = {
            enable = true,
            ten = false,
            hideui = false,
            },
          reputation = {
            enable = true,
            onlyexalted = false,
            hideui = false,
            },
          bg = {
            enable = true,
            onlywin = false,
            hideui = false,
            },
          dungeon = {
            enable = true,
            onlyhero = false,
            hideui = false,
            },
          boss = {
            enable = true,
            ten = true,
            tenhero = true,
            twentyfive = true,
            twentyfivehero = true,
            five = true,
            fivehero = true,
            world = true,
            hideui = false,
            },
          statistic = {
            enable = true,
            achievement = 0,
            levelup = 0,
            reputation = 0,
            battleground = 0,
            dungeon = 0,
            boss = 0,
            user = 0,
            fails = 0,
            }
          },
        global = {
          blacklist = AchScreenshotter2:getblacklist(),
          whitelist = {}
          }
        }
return defaults
end

function AchScreenshotter2:getmainoptions()
  local version = GetAddOnMetadata(addonname, "X-Curse-Packaged-Version") or ""
  local mainoptions = {
        name = L["Main"].." "..version,
        type = "group",
        --desc = L["options desc"],
        args = {
          addon_enable = {
            name = L["enable"],
            type = "toggle",
            order = 1,
            desc = L["enable desc"],
            get = function() return mod.sv.enable end,
            set = function(_, val)
              mod.sv.enable = val
              mod:RefreshConfig()
              end,
            },
          waittime = {
            name = L["waiting time"],
            type = "range",
            order = 2,
            desc = L["waiting time desc"],
            min = 1,
            max = 100,
            bigStep = 5,
            get = function() return mod.sv.waittime end,
            set = function(_, val)
              mod.sv.waittime = val
              end,
            },
          delaytime = {
            name = L["delay"],
            type = "range",
            order = 3,
            desc = L["delay desc"],
            min = 0,
            max = 5,
            bigStep = 0.01,
            get = function() return mod.sv.delay end,
            set = function(_, val)
              mod.sv.delay = val
              end,
            }
          }
        }
return mainoptions
end

function AchScreenshotter2:getstatisticoptions()
  local version = GetAddOnMetadata(addonname, "X-Curse-Packaged-Version") or ""
  local statisticoptions = {
        name = L["Statistic"].." "..version,
        type = "group",
        --desc = L["statistic options"],
        args = {
          statistic = {
            name = L["statistic enable"],
            type = "toggle",
            order = 71,
            desc = L["statistic enable desc"],
            get = function() return mod.sv.statistic.enable end,
            set = function(_, val)
              mod.sv.statistic.enable = val
              end,
            },
          achievement = {
            name = function() return L["achievement statistic desc:"] .." ".. mod.sv.statistic.achievement end,
            type = "description",
            order = 72,
          },
          levelup = {
            name = function() return L["levelup statistic desc:"] .." ".. mod.sv.statistic.levelup end,
            type = "description",
            order = 73,
          },
          reputation = {
            name = function() return L["reputation statistic desc:"] .." ".. mod.sv.statistic.reputation end,
            type = "description",
            order = 74,
          },
          battleground = {
            name = function() return L["battleground statistic desc:"] .." ".. mod.sv.statistic.battleground end,
            type = "description",
            order = 75,
          },
          dungeon = {
            name = function() return L["dungeon statistic desc:"] .." ".. mod.sv.statistic.dungeon end,
            type = "description",
            order = 76,
          },
          boss = {
            name = function() return L["boss statistic desc:"] .." ".. mod.sv.statistic.boss end,
            type = "description",
            order = 77,
          },
          user = {
            name = function() return L["user statistic desc:"] .." ".. mod.sv.statistic.user end,
            type = "description",
            order = 78,
          },
          fails = {
            name = function() return L["fails statistic desc:"] .." ".. mod.sv.statistic.fails end,
            type = "description",
            order = 79,
          },
          reset = {
            name = L["reset statistic"],
            type = "execute",
            order = 80,
            desc = L["reset statistic desc"],
            func = function()
              mod.sv.statistic.achievement = 0
              mod.sv.statistic.levelup = 0
              mod.sv.statistic.reputation = 0
              mod.sv.statistic.battleground = 0
              mod.sv.statistic.dungeon = 0
              mod.sv.statistic.boss = 0
              mod.sv.statistic.user = 0
              mod.sv.statistic.fails = 0
            end,
          },
        }
      }
return statisticoptions
end 